use Test::More;

use strict;
use warnings;

plan tests => 9;

use_ok ('MooseX::Meta::Parameter::Moose');

use_ok ('MooseX::Meta::Parameter::Moose');

use_ok ('MooseX::Meta::Signature::Named');

use_ok ('MooseX::Meta::Signature::Named::Compiled');

use_ok ('MooseX::Meta::Signature::Positional');

use_ok ('MooseX::Meta::Signature::Positional::Compiled');

use_ok ('MooseX::Meta::Signature::Combined');

use_ok ('MooseX::Meta::Signature::Combined::Compiled');

use_ok ('MooseX::Method');

